//----------------------------------------------------------------------------
//
//                     XMM Science Analysis System (SAS)
//                   (c) 1997-2010 European Space Agency
//
//---------------------------------------------------------------------------
//
//	Random.h:	--- An abstract I/F to a random number generator
//
//	DESCRIPTION:
//		Concrete instances of the provided class Random shall be used
//		to get uniformily distributed pseudo-random numbers in the
//		interval [0, 1].
//
//	CHANGE HISTORY:
//		UL, 1999-05-27: 1.0.0: - first version
//		UL, 1999-06-08: 1.1.0: - added min()/max() methods
//		UL, 2002-03-02: 1.2.0: - extented class to give client control
//								 over type of range returned by rand()
//							   - added pushState()/popState()
//		UL, 2002-03-25: 1.3.0: - added second flavor of pushState() +
//								 minMaxRangeType()
//
//	AUTHOR:
//		U. Lammers (UL), Thu May 27 16:15:23 MET DST 1999
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/utils/src/Random.h,v 1.7 2008/09/26 16:38:55 eojero Exp $
//

#ifndef RANDOM_H
#define RANDOM_H

#include <vector>

class RandomNumberGenerator {
public:
	enum SophisticationLevel { QUICK_AND_DIRTY, GOOD, REALLY_GOOD,
							   ALMOST_PERFECT, PERFECT };
	enum RangeType { OPEN, OPEN_CLOSED, CLOSED_OPEN, CLOSED };

	RandomNumberGenerator(SophisticationLevel level = ALMOST_PERFECT,
						  RangeType rangeType = CLOSED,
						  unsigned seed = 0, double min = 0., double max = 1.)
					 : _level(level), _rangeType(rangeType), _seed(seed),
					   _min(min), _max(max) {}
		//
		//	PURPOSE:
		//		Create a random number generator object.
		//
		//	ARGUMENTS:
		//		level	: Describes the level of sophistication
		//				  of the generator in terms of quality of the
		//				  returned random numbers, i.e. long periodicities
		//				  and passing of standard statistical tests;
		//				  Concrete implementations of this class are
		//				  free to ignore the SophisticationLevel.
		//		rangeType: Parameter controlling the type of range of
		//				  random numbers that rand() (see below)
		//				  shall return:
		//					OPEN:			min <  rand() <  max
		//					OPEN_CLOSED	:	min <  rand() <= max
		//					CLOSED_OPEN	:	min <= rand() <  max
		//					CLOSED		:	min <= rand() <= max
		//		seed	: Seed value controlling the sequence
		//				  of pseudo-random numbers; identical
		//				  seed values are guaranteed to yield
		//				  the same sequence of pseudo random
		//				  numbers
		//		min/max	: lower/upper bound of numbers returned by
		//				  rand() (see below)
		//

	SophisticationLevel sophisticationLevel() const { return _level; }
	RangeType rangeType() const { return _rangeType; }
	unsigned seed() const { return _seed; }
		//
		//	PURPOSE:
		//		Return the current values of the sophistcation level, range
		//		type and seed which are in effect.
		//

	virtual void sophisticationLevel(SophisticationLevel level) = 0;
	virtual void rangeType(RangeType rangeType) = 0;
	virtual void seed(unsigned seed) = 0;
		//
		//	PURPOSE:
		//		Change the sophisticationLevel/rangeType/seed of the generator
		//

	double min() const { return _min; }
	double max() const { return _max; }
		//
		//	PURPOSE:
		//		Return the current values of the lower/upper range of
		//		numbers returned by rand()


	virtual void min(double minVal) = 0;
	virtual void max(double maxVal) = 0;
	virtual void minMaxRange(double minVal, double maxVal,
							 RangeType rangeType) = 0;
		//
		//	PURPOSE:
		//		Define lower/upper bound + range type of numbers returned by
		//		rand() below
		//
		//	ARGUMENTS:
		//		minVal/maxVal	: lower/upper bound of numbers returned
		//						  by rand()
		//		rangeType		: type of random number interval (see above)
		//


	void pushState();
	void pushState(double minVal, double maxVal, RangeType rangeType)
		{ pushState(); minMaxRange(minVal, maxVal, rangeType); }
	void popState();
		//
		//	PURPOSE:
		//		Push/pop state of random number generator onto/from a
		//		stack; popping if stack is empty has no effect;
		//		Second form of pushState() is equivalent to calling sequence:
		//			pushState();
		//			minMaxType(min, max, rangeType);
		//


	virtual void reset() = 0;
		//
		//	PURPOSE:
		//		Reset the generator to the state it was in right after
		//		construction
		//


	virtual double rand() = 0;
		//
		//	PURPOSE:
		//		Return a uniformily distributed pseudo-random number in the
		//		range
		//			o (min, max)
		//			o (min, max]
		//			o [min, max)
		//			o [min, max]
		//		depending on the value of <rangeType> specified upon
		//		object construction or set via the dedicated method
		//		rangeType(RangeType rt)
		//

	virtual ~RandomNumberGenerator() { }
		//
		//	PURPOSE:
		//		destroy the object constructed with the above constructors
		//

protected:
	SophisticationLevel	_level;
	RangeType			_rangeType;
	unsigned			_seed;
	double				_min;
	double				_max;

private:
	class State {
	public:
		State(RangeType rt, double min, double max)
			: _rt(rt), _min(min), _max(max) {}
		RangeType _rt;
		double	  _min, _max;
	};
	std::vector<State> _stateVec;
};

#define randGen		randGenFunc()
RandomNumberGenerator &randGenFunc();

#endif
