//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997 - 2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	SimpleToken.cc	--- implements the SimpleToken class which represents
//                          basic string tokens.
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/utils/src/SimpleToken.cc,v 1.7 2008/09/26 16:38:55 eojero Exp $
///


#include <map>
#include <string>

#include "Tracer.h"
#include "SimpleToken.h"


class SimpleTokenValue
{
	public :
        SimpleTokenValue() {}
		virtual ~SimpleTokenValue() {}
		virtual char asChar() const = 0;
		virtual std::string asString() const = 0;

	protected :
		void invalidOperation() const { PROC std::cerr << "invalid operation" << std::endl; exit( 1 ); }
};



class SimpleTokenStringValue : public SimpleTokenValue
{
	public :
		SimpleTokenStringValue( const std::string& v ) : _mydata( v ) { PROC ENDPROC }
		virtual ~SimpleTokenStringValue() { PROC ENDPROC }
		virtual std::string asString() const { PROC RETURN _mydata; }
		virtual char asChar() const { PROC invalidOperation(); RETURN ' '; }
	private :
		std::string	_mydata;
};


class SimpleTokenCharValue : public SimpleTokenValue
{
	public :
		SimpleTokenCharValue( char v ) : _mydata( v) { PROC ENDPROC }
		virtual ~SimpleTokenCharValue() { PROC ENDPROC }
		virtual char asChar() const { PROC RETURN _mydata; }
		virtual std::string asString() const { PROC invalidOperation(); RETURN ""; }
	private :
		char	_mydata;
};



SimpleToken::SimpleToken() : _value(0), _id( SimpleToken::Undefined )
{
	PROC
	ENDPROC
}


SimpleToken::SimpleToken( const SimpleToken& org ) : _value( 0 ), _id( org.id() )
{
	PROC
	switch( _id )
	{
		case SimpleToken::Undefined :
			break;
		case SimpleToken::Character :
			_value = new SimpleTokenCharValue( org.asChar() );
			break;
		case SimpleToken::String :
			_value = new SimpleTokenStringValue( org.asString() );
			break;
		case SimpleToken::Eof :
			break;
		default :
			std::cerr << "Unhandled case" << std::endl;
			exit( 1 );
	} // switch
	ENDPROC
}


SimpleToken&
SimpleToken::operator=( const SimpleToken& org )
{
	PROC
	delete _value;
	_value = 0;
	_id = org.id();
	switch( _id )
	{
		case SimpleToken::Undefined :
			break;
		case SimpleToken::Character :
			_value = new SimpleTokenCharValue( org.asChar() );
			break;
		case SimpleToken::String :
			_value = new SimpleTokenStringValue( org.asString() );
			break;
		case SimpleToken::Eof :
			break;
		default :
			std::cerr << "Unhandled case" << std::endl;
			exit( 1 );
	} // switch

	RETURN *this;
}


SimpleToken::SimpleToken( char v ) : _value( new SimpleTokenCharValue( v ) ), _id( SimpleToken::Character )
{
	PROC
	ENDPROC
}


SimpleToken::SimpleToken( const std::string& v ) : _value( new SimpleTokenStringValue( v ) ), _id( SimpleToken::String )
{
	PROC
	ENDPROC
}


SimpleToken::SimpleToken( Id id ) : _value( 0 ), _id( id )
{
	PROC
	switch( _id )
	{
		case SimpleToken::Eof :
		case SimpleToken::Undefined :
			break;
		case SimpleToken::Character :
			_value = new SimpleTokenCharValue( '\0' );
			break;
		case SimpleToken::String :
			_value = new SimpleTokenStringValue( "" );
			break;
		default :
			std::cerr << "Unhandled case" << std::endl; exit( 1 );
			break;
	} // switch
	ENDPROC
}


SimpleToken::~SimpleToken()
{
	PROC
	delete _value;
	_value = 0;
	ENDPROC
}


SimpleToken::Id
SimpleToken::id() const
{
	PROC
	RETURN _id;
}


void
SimpleToken::id( Id id )
{
	PROC
	*this = SimpleToken( id );
	ENDPROC
}


void
SimpleToken::value( char v )
{
	PROC
	*this = SimpleToken( v );
	ENDPROC
}


void
SimpleToken::value( const std::string& v )
{
	PROC
	*this = SimpleToken( v );
	ENDPROC
}


char
SimpleToken::asChar() const
{
	PROC
	RETURN _value -> asChar();
}


std::string
SimpleToken::asString() const
{
	PROC
	RETURN _value -> asString();
}


SimpleToken&
SimpleToken::operator=( char v )
{
	PROC
	delete _value;
	_id = SimpleToken::Character;
	_value = new SimpleTokenCharValue( v );
	RETURN *this;
}


SimpleToken&
SimpleToken::operator=( const std::string& v )
{
	PROC
	delete _value; _value=0;
	_id = SimpleToken::String;
	_value = new SimpleTokenStringValue( v );
	RETURN *this;
}


SimpleToken&
SimpleToken::operator=( Id id )
{
	PROC

	delete _value;
	_value = 0;
	_id = id;

	switch( _id )
	{
		case SimpleToken::Eof :
		case SimpleToken::Undefined :
			break;
		case SimpleToken::Character :
			_value = new SimpleTokenCharValue( '\0' );
			break;
		case SimpleToken::String :
			_value = new SimpleTokenStringValue( "" );
			break;
		default :
			std::cerr << "Unhandled case" << std::endl; exit( 1 );
	} // switch

	RETURN *this;
}


static struct
{
	SimpleToken::Id	id;
	const char*		str;
} ids[] = {
	{ SimpleToken::Undefined, 	"Undefined" },
	{ SimpleToken::Character, 	"Character" },
	{ SimpleToken::String, 		"String" },
	{ SimpleToken::Eof, 		"Eof" }};


std::string
SimpleToken::idAsString() const
{

	static bool				firstTime = true;
	static std::map< SimpleToken::Id, const char*>	stringOf;

	if( firstTime )
	{
		const unsigned int nr = sizeof( ids ) / sizeof( ids[ 0 ] );

		for( unsigned int i = 0; i < nr; ++i )
		{
			stringOf[ ids[ i ].id ] = ids[ i ].str;
		} // for
		firstTime = false;
	} // if
	RETURN stringOf[ _id ];
}


