/*****************************************************************************
 *
 *  stringFns.h                                 XMM Science Analysis Subsystem
 *
 *  string functions
 *
 *  (c) 1999 ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: StringFns.h,v 1.6 2008/09/26 16:38:55 eojero Exp $
 *
 *****************************************************************************/

#ifndef STRINGFNS_H
#define STRINGFNS_H

#include <string>       // C++ headers
#include <list>
#include <cstdlib>

class StringFns
{
public:

    /*
     *  Split a string into tokens. The argument 'separator' is a list of
     *  separator characters. Multiple consecutive separator charaters in
     *  the string are treated as a single separator. Leading and trailing
     *  separator characters are ignored.
     *
     *  Example:
     *    splitString("ooxyaaapqor", "oa") -> ["xy","pq","r"]
     */
    static std::list<std::string> splitString(const std::string& s,
										  const std::string& separator=" \t");

    /*
     *  Split a quoted string into tokens. Handles both single and double
     *  quotes (but not nested). Returns false via argument 'ok' if the
     *  quotes are mismatched.
     *
     *  Example:
     *    splitStringList("'a b' 'c d'",ok) -> ["a b", "c d"]
     */
    static std::list<std::string> splitStringList(const std::string& str,
												  bool* ok=0);


    /*
     *  Split a string into tokens. The argument 'delim' is a list of
     *  delimiter characters. Multiple consecutive separator charaters in
     *  the string produce tokens containing the empty string.
     *
     *  Example:
     *    splitDelimitedString("a|b|c", "|") -> ["a","b","c"]
     *    splitDelimitedString("a||c", "|")  -> ["a","","c"]
     *    splitDelimitedString("a| |c", "|") -> ["a"," ","c"]
     *    splitDelimitedString("a|", "|")    -> ["a"," "]
     */

    static std::list<std::string> splitDelimitedString(const std::string& s,
					     const std::string& delim=",");

    /*
     *  Parse an integer. If valid, return true and pass result via argument 
     *  'result'. Trailing garbage is not allowed.
     *
     *  Examples:
     *    ok = parseInt("123",  n) -> ok==true, n==123
     *    ok = parseInt("12.3", n) -> ok==false, n unchanged
     */
    static bool parseInt(const std::string& str, int& result);

    /*
     *  Parse a double. If valid, return true and pass result via argument 
     *  'result'. Trailing garbage is not allowed.
     *
     *  Examples:
     *    ok = parseDouble("12.3e2",  n) -> ok==true, n==1230
     *    ok = parseDouble("12x", n) -> ok==false, n unchanged
     */
    static bool parseDouble(const std::string& str, double& result);
};

#endif
