//
//	STimetest.cc --- test class STime
//
//	DESCRIPTION:
//		Intended to test the correctness of implementation of class STime;
//
//	INVOCATION:
//		stimetest
//
//	RETURNS:
//		0	 : all tests passed successfullly
//		!=0	 : at least one test failed; corresponding message printed to std::cerr
//
//	CHANGE HISTORY:
//		UL, 29/10/97: 1.0.0: - first version
//
//	AUTHOR:
//		U. Lammers, Wed Oct 29 17:37:06 WET 1997
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/utils/test/stimetest.cc,v 1.21 2008/09/26 16:38:55 eojero Exp $
//  

#include <cassert>
#include <iostream>
#include <iomanip>
#include <string>
#include <cmath>

#include "STime.h"
#include "common.h"

using namespace std;

int main()
{
	STime::errorHandling = STime::EXCEPTION;
	try {

	// default construction - object should be initialized to
	// STime::earliest
	STime t1;
	assert(t1 == STime::earliest);

	// construction from year/month/day/hour/min/sec tuple
	std::string t2string("1997-10-23T14:15:34.123");
	STime t2(1997, 10, 23, 14, 15, 34, .123);
	assert(t2.toString(3, STime::FITS) == t2string);

	// construction from string object
	STime t3(t2string);
	assert(t3.toString(3, STime::FITS) == t2string);
	assert(t3.toString(0) == t2string.substr(0, t2string.length()-4));
	for (unsigned n=1; n<=3; ++n)
		assert(t3.toString(n) == t2string.substr(0, t2string.length()-3+n));

	std::string t4string("Wed Oct 29 15:37:19 1997");
	STime t4(t4string);
	assert(t4.toString(0, STime::CALENDAR) == t4string);

	// construction from MJD/JD/DYN number
	// conversion to MJD/JD number
	std::string t5string("1997-10-13T00:00:00.000");
	double t5julday = 2450734.5;
	STime t5(t5julday, STime::JD);
	assert(t5.toString() == t5string);
	assert(t5.julianDay().full() == t5julday);

	double t6mjulday = t5julday - 2400000.5;
	STime t6(t6mjulday, STime::MJD);
	assert(t6.toString() == t5string);
	assert(t6.modifiedJulianDay().full() == t6mjulday);

	STime t51(1998.5, STime::DYN);
	std::string t51string("1998-07-02T12:00:00.000");
	assert(t51.toString() == t51string);
	assert(t51.decimalYearNumber() >= 1998.499999 &&
		   t51.decimalYearNumber() <= 1998.500001);

	// copy constructor
	STime t7 = t5;
	assert(t7.toString() == t5string);

	// assignment
	t7 = t2;
	assert(t7.toString() == t2string);

	// +=, -=
	std::string t8string = "1999-12-31T23:59:00.100";
	std::string t8string2 = "2000-01-01T00:04:03.100";
	STime t8(t8string);
	double t8offset = 5. * STime::secondsPerMinute + 3.;
	t8 += t8offset;
	assert(t8.toString() == t8string2);
	t8 -= t8offset;
	assert(t8.toString() == t8string);

	// -, +, differnce between time instances
	STime t9 = t8 + t8offset;
	assert(t9.toString() == t8string2);
	double t9mt8 = t9 - t8;
	assert((long)(t9mt8 * 1000000. +.5) == (long)(t8offset * 1000000. + .5));

	STime t10 = t9 - 10. * STime::secondsPerDay;	// t9 - 10 days
	std::string t10string = "1999-12-22T00:04:03.100";
	assert(t10.toString() == t10string);

	// binary operators
	STime t11;
	t11 = t10;
	assert(t11 == t10);
	t11 -= 0.001;
	assert(t11 < t10);
	assert(t11 <= t10);
	assert(t11 != t10);

	t11 += 0.002;
	assert(t11 > t10);
	assert(t11 >= t10);
	assert(t11 != t10);

	std::string t12string = "1997-12-01T08:30:00.000";
	STime t12(t12string);
	assert(t12.toString() == t12string);

	//std::cout << "This is the XMM mission reference time ->" <<
	//		XMM::missionReferenceTime << std::endl;
	//assert(XMM::missionReferenceTime >= STime::earliest);
	//assert(XMM::missionReferenceTime <= STime::latest);
	//assert(XMM::missionReferenceTime - XMM::missionReferenceTime == 0.);

	//
	//	check if leap seconds are correctly handled
	//
	std::string t13string = "1995-12-31T23:59:59.100";
	STime t13(t13string);
	assert(t13.toString() == t13string);
	t13 += 1.;	//	within a leap second
	assert(t13.toString() == "1995-12-31T23:59:60.100"); // within a leap sec
	t13 += 1.;	//	after the leap seond
	assert(t13.toString() == "1996-01-01T00:00:00.100");
	assert(t13.leapSeconds() == 30);

	STime t14("1994-01-01"), t15("1997-06-30T23:59:59.1");
	double diff = t15-t14;
	assert(diff >= 110332801.1 && diff <= 110332801.100001);	//	before leap
	diff = t15+1 - t14;
	assert(diff >= 110332802.1 && diff <= 110332802.100001);	//	during leap
	diff = t15+2 - t14;
	assert(diff >= 110332803.1 && diff <= 110332803.100001);	//	after leap

	STime t16("1998-01-01");
	diff = t16-t14 -
			((4 * 365 + 1) * 86400. + t16.leapSeconds() - t14.leapSeconds());
	assert(abs(diff) <= 1.e-6);

	//	time differences across leap seconds
	STime t17("1988-01-01"), t18("1987-12-31T23:59:59");
	for (double t=-2.; t<0.; t+=.1, t18+=.1) {
		double td = t18 - t17;
		assert(td < 0. && abs(td/t-1.) <= 1.e-6);
	}

	//	a few precision related tests
	std::string t22str("1999-12-31T23:59:59.999");
	STime t22("1999-12-31T23:59:59.999");
	assert(t22.toString() == t22str);
	STime t23("1999-12-31T23:59:59.99999");
	assert(t22.toString() == t22str);

	STime t25("2001-01-01T00:00:59.999999999");
	assert(t25.toString(5) == "2001-01-01T00:00:59.99999");
	STime t24("1999-12-31T23:59:59.999999999");
	assert(t24.toString(6) == "1999-12-31T23:59:59.999999");

	const char *testTimes[] = { "2000-04-11T21:57:00",
								"2000-04-11T21:55:00",
								"2000-04-11T21:53:00",
								"2000-04-11T21:49:00" };
	for (unsigned n=0; n<NUM_OF(testTimes); ++n)
		assert(STime(testTimes[n]).toString(0) == testTimes[n]);

	std::cout<<"This should be the current GMT ->"
		<<STime::currentTime()<<std::endl;
	std::cout<<"Again                          ->"
		<<STime("now")<<std::endl;
	std::cout<<"Local time is                  ->"
		<<STime::currentTime(false)<<std::endl;
	assert(STime::currentTime() == STime("now") ||
		   STime::currentTime() == STime("now") ||
		   STime::currentTime() == STime("now"));

	} catch (STime::InvalidTime &iv) {
		iv.error();
		return 1;
	}

	assert(STime::isValidTime("1999-01-01T00:00:00.123"));
	assert(STime::isValidTime("   1999-01-01T00:00:00.123    "));
	assert(!STime::isValidTime("1999-01-01T"));
	assert(!STime::isValidTime("XXXXXXXX"));
	assert(!STime::isValidTime("1999-01-01T00:00:61"));
	assert(!STime::isValidTime("             "));

	assert(!STime::isValidTime(-1.));

	assert(STime::isValidTime(2000, 2, 29));
	assert(!STime::isValidTime(1900, 2, 29));

	assert(!STime::isValidTime(1999, 8, 8, 24, 0, 0, 0.));

	double TWO_PI = 2.*acos(-1.);

	//
	//	sidereal time in Berlin (longitude=+13.3deg) at 1982-04-15T20:00 UTC
	//	is 10:27:48
	//	see "Fundamental Astronomy", p. 42
	//
	STime t20("1982-04-15T20:00:00");
	double sidBerlin = t20.siderealTime(13.3*TWO_PI/360.);
	double secs = sidBerlin/TWO_PI*86400.;
	double mins;
	secs = 60.*modf(secs/60., &mins);
	double hours;
	mins = 60.*modf(mins/60., &hours);
	int ih = INT(hours+.5),
		im = INT(mins+.5),
		is = INT(secs+.5);
	std::cout<<"Sidereal time in Berlin on "<<t20<<" GMT is: "
		<<ih<<':'<<im<<':'<<is<<std::endl;
	assert(ih==10 && im==27 && is==48);

	STime t21("2000-07-08T09:44:30");
	double sidTime = t21.siderealTime(-80.38216389*TWO_PI/360.);
	std::cout<<"Sidereal time: "<<setprecision(10)<<360.*sidTime/TWO_PI
			 <<std::endl;

	std::cout << "\nAll tests sucessfully passed\n";

	return 0;
} // main
